;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F676. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ZU676_41.asm                                           *
;    Date:                                                            *
;    File Version:   v4.1                                                 *
;                                                                     *
;    Author:       NickF                                                   *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F676.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************

	list      p=16f676           ; list directive to define processor
		


	#include <p16F676.inc>        ; processor specific variable definitions
	#include <macro43.inc>
	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;***** VARIABLE DEFINITIONS

	CBLOCK 0x20			;      

D5,D4,D3,D2,D1,flag,count,t_zn
znk,cyc_tim,t_cyc,NumH,NumL,TEMP1,TEMP0
znak,zn_dac,T1H,T1L,reg3,ad_tab				;NumH,L-HEX , D(n) - DEC    
Uzap:2,Upaz:2,Izap:2,Ipaz:2,Tzap:2,Tpaz:2					;"0" - , "1" - , "2" - 
;"3" - , "4" -  , "5" -  Q = t*I * 

		ENDC
;flag -   <1,0>    
; "7" -      "1",   "0"  
; "6" -    "1"    "0" 
; "5" -    "1" - Li, "0" - Hi
; "4" -   "0"   "1" 
; "3" -    "0"     "1" 
; "2" -   "1"
; "1" -   "1"   "0"
; "1"  "1"   "0"
; "2" -    
; "3" -  : ., , , ,  , 
;znak - 7-  , znk -  1,2,3
;count,t_zn -      /
;cyc_tim -   TMR0,    
;t_cyc -    ,   / "button"
;NumH     ADRESH, NumL    ADRESL
;T1H,T1L -    TMR1H  TMR1L


;share variables
;=B4*104 + B3*103 + B2*102 + B1*101 + B0*100
;=B4*10000 + B3*1000 + B2*100 + B1*10 + B0
;D5     	B4
;D4     	B3
;D3     	B2
;D2     	B1
;D1			B0

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program

		ORG     0x004             ; interrupt vector location
		goto	ObrInt
		org		0x7
; 1 :    ->  
;    ,   W=07h -  "7"
Segm7		
    CLRF   PCLATH
	ADDWF  PCL, F
;		 0	  1	   2	3	 4	  5	   6	7	 8	  9	   A	b	 C	  d	   E	F
	dt	07E, 00C, 0B6, 09E, 0CC, 0DA, 0FA, 00E, 0FE, 0DE, 0EE, 0F8, 072, 0BC, 0F2, 0E2 
;-    -  
; 2    W=0fh -  , W=012h -  
	dt	0EC, 008, 080	;"Hi-"  0fh
	dt	070, 008, 080	;"Li-"  012h
	dt	080, 09E, 080	;"-3-"	015h	
	dt	080, 07C, 080	;"-U-"  018h
	dt	080, 0E6, 080	;"-P-"	01bh
	dt	080, 00C, 080	;"-I-"	01eh	
	dt	080, 0F0, 080	;"-t-"  021h
	dt	07E, 09E, 080	;"O3-"	024h	
	dt	07E, 0E6, 080	;"OP-"  027h
	dt	080, 06E, 080	;"--"	02ah
	dt	080, 072, 080	;"-C-"  02dh
	dt	07E, 07E, 07E	;"000"  030h
	dt	00C, 00C, 00C	;"111"	033h
ULz		equ	0x0348	;Uzap max = 4.20 
ULp		equ	0x0258	;Upaz max = 3.00 
UHz		equ	0x012c	;Uzap max = 1.50 
UHp		equ	0x00c8	;Upaz max = 1.00 

;*******************************************************************

main
		; init PORTA & SFR

		bcf 	STATUS,RP0 			;Bank 0
		bsf		STATUS,NOT_TO		;reset on POR
		bsf		STATUS,NOT_PD
		clrf 	PORTA 				;Clear PORTA
		movlw 	07h 				;Set RA<2:0> to 
		movwf 	CMCON 				;comp-off digital I/O
		bsf 	STATUS,RP0 			;Bank 1
		bsf		PCON,NOT_POR		;enable POR
		clrf 	ANSEL 				;digital I/O
		clrf 	WPUA 				;  
		movlw 	043h
		movwf 	TRISA				;Set PORTA<5,1.0> as input
		movlw 	03h
		movwf 	ANSEL 				;Set PORTA <1.0>  analog input
		clrf 	PIE1 				;   		

;init PORTC
		bcf 	STATUS,RP0 			;Bank 0
		clrf	PORTC
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	TRISC 				;set PORTC as outputs
		bcf 	STATUS,RP0 			;Bank 0

; these first 4 instructions are not required if the internal oscillator is not use
		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0

; ADC,TMR0
	banksel		ADCON1			
		movlw	030h			;    
		movwf 	ADCON1			;RC  ~ 8 us.
		clrf	TMR0			;  "0"
		movlw	82h				; - 
	banksel		OPTION_REG		;   1/8 
		movwf	OPTION_REG		; CLK  TMR0 - 8 ,   "T0CS"=0
;   256*8=2.048 .
;  1
		movlw	06h				;   1/1 
	banksel		T1CON			;    TMR1
;   32768 ,    PA5
		movwf	T1CON
	banksel		PIE1
		bsf		PIE1,TMR1IE		;   TMR1
; remaining code goes here
start
;  020h-04fh
		movlw	020h			;   
		movwf	FSR
c_ind	clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		comp	060h,FSR,c_ind			;FSR  60h? 
;,     

;  
		movlw	6
		movwf	cyc_tim			;   6
		movlw	018h			; "-U-"
		movwf	ad_tab			;  
		banksel		T1CON
			clrf	TMR1L
			clrf	TMR1H
			bsf		T1CON,0		;start TMR1
start1		movlw	1
		movwf	znk				; 1 
		
;	
start_adc
; 
		bcf 	STATUS,RP0 		;Bank 0
		clrf	INTCON
		bsf		INTCON,PEIE		;   
		bsf		INTCON,T0IE		;    TMR0
		bsf		INTCON,GIE		; 
			comp	0,reg3,st01			;  
			sbit	6,flag,st_sim		;  
			sbit	3,flag,stUp			;   
			sbit	4,flag,stHi			
			ust_wri		ULz				; ,  Li
			goto		stm
stHi		ust_wri		UHz				; ,  Hi
			goto		stm
stUp		sbit	4,flag,stHip
			ust_wri		ULp				; ,  Li
			goto		stm
stHip		ust_wri		UHp				; ,  Hi
			goto		stm
st01		comp	1,reg3,st02			;  
			sbit	5,flag,stm
			goto	st_sim
st02		comp	2,reg3,st03
			sbit	4,flag,st_sim

			goto	stm
st03		goto	stm
st_sim		sbit	0,D5,st_tab
			goto	stm
st_tab		tab	ad_tab
		

;      4-   
stm		call	Mult			; /     5
		call	Bin2DecFast		; /  HEX -> DEC
; 2  /      

		comp	1,reg3,st0		;   
; 
		movf	D3,w		; ,  "0123"  "123"
		movwf	D4
		movf	D2,w	
		movwf	D3
		movf	D1,w	
		movwf	D2
		goto	stt
; 
st0		clrc				; "D1"
		comp1	4,D1,stt	; "D1" > 4,   1   
		incf	D2			;  ,   "9,99" 

;    			
stt		movlw	020h		;   
		addwf	znk,w
		movwf	FSR
		movf	INDF,w		;      
;      1  2
st_seg
		call	Segm7		;/    7  
;      
		movwf	znak		;akk->znak
		call	WriteZnak	;/    

		call	Button		;  ,   /  
		movlw	.10
		call	Pause
			comp	3,znk,lost	; "znk" = 3,   
		goto	loo
lost		comp	0,reg3,loop
			sbit	6,flag,loop
			sbit	4,flag,loHi
			sbit	3,flag,loz
			wri_res		ULp
			goto	loop
loz			wri_res		ULz
loHi		sbit	3,flag,loh
			wri_res		UHp
			goto	loop
loh			wri_res		UHz
			goto	loop
loo		call	AdcPusk
loop	comp	0,cyc_tim,loop
		movlw	6	
		movwf	cyc_tim

		incf	znk,f
		comp	4,znk,st_ind

;   ,     ,    

		goto	start1
st_ind		comp	0,D5,stt
			goto	st_tab
;****************************
;
;****************************
;/     LED 
;  /   znak 
;7-      
WriteZnak
		clrf	t_zn
		movf	znk,w		; "znk"  "w"
		movwf	TEMP0
		comp	0,D5,ww
		goto	wc
ww		comp	0,reg3,wt		;   
		comp	1,znk,wt		;
		bsf		znak,0		;znk = 1,  
		goto	wc
wt		comp	2,reg3,wc
		comp	2,znk,wc
		bsf		znak,0		;znk = 2,  
	; ,   
wc		bsf		STATUS,C		
shi		rrf		t_zn,f		;     
		decfsz	TEMP0,f		;     
		goto	shi
		movf	t_zn,w
		xorwf 	zn_dac,w	;  "zn_dac"  "t_zn"
;*******************************
;		xorlw	0f0h		; ,     
;*******************************
		movwf	TEMP1		;    "zn_dac"
		movf	znak,w
;*******************************
		xorlw	0ffh		; ,     
;*******************************
		movwf	TEMP0		;    "znak"
		call	Shift_16bit
		return
Pause
		movwf	TEMP0
pa		decf	TEMP0,f		
		skpz
		goto	pa 
		return
;/  
Button
		btfss	PORTA,3
		goto	bt_out1			;  , 
		btfss	flag,7			;  
		goto	bt0
		decfsz	t_cyc,f			;   36*8 
		goto	bt_out1			; 0, 
		bcf		flag,7
		bcf		T1CON,TMR1ON		; TMR1
		clrf	TMR1L
		clrf	TMR1H
;   
		comp	3,znk,bt2		;   3	
		incf	reg3
		comp	7,reg3,bt30
		clrf	reg3			;   
;		clrf	flag			;Hi - ,  
bt30		comp	0,reg3,bt31
			adt		018h
bt31		comp	1,reg3,bt32
			adt		01eh
bt32		comp	2,reg3,bt33
			adt		021h		
bt33		comp	3,reg3,bt34
			adt		015h
bt34		comp	4,reg3,bt35
			adt		0fh	
bt35		comp	5,reg3,bt36
			adt		024h
bt36		comp	6,reg3,bt_out
			adt		02dh			
				
bt2		comp	2,znk,bt1			;   2		
		comp	1,reg3,btu			;,   ?
		sbit	5,flag,bt_out1			;, 
		comp1	01fh,zn_dac,bt21			;,  ?
;,     
		clrf	zn_dac				;,  
		goto	bt_out1
bt21	incf	zn_dac,f			;  
		goto 	bt_out1
btu		sbit	6,flag,bt_out1
		comp	0,reg3,bt_out1
		goto	bt_out
bt1		comp	1,znk,bt_out1		;   1		
		comp	0,reg3,bt01				
		inv1	6,0x30,0x33,bt_out		;  "6" 
bt01	comp	1,reg3,bt02			
		inv1	5,0x30,0x33,bt_out		;  "5" 
bt02	comp	2,reg3,bt03		
		inv1	4,0x30,0x33,bt_out		;  "4" 
bt03	comp	3,reg3,bt04			
		inv1	3,015h,01bh,bt_out		;  "3" -
bt04	comp	4,reg3,bt05			
		inv1	2,0fh,012h,bt_out		;  "2" Hi-Li
bt05	comp	5,reg3,bt06			
		inv1	1,0x24,0x27,bt_out		;  "1"  -
bt06	comp	6,reg3,bt_out		
		inv1	0,0x2d,0x2a,bt_out		;  "0" -
bt0		bsf		flag,7				;  
		movlw	8
		movwf	t_cyc
		goto	bt_out1
bt_out		bsf		T1CON,TMR1ON		; TMR1
			clrf	T1L
			bcf		D5,0
bt_out1		return

;copy 13-bit number to temporary
;12-bit code
;Posted by Peter Hemsley on September 18, 2004 at 17 40 56 
;Message   ; Bonus 12-Bit version
  ; Leaner and Meaner, 36 instructions
  ; Ideal for displaying A/D values
Bin2DecFast
		movf    NumH,w
        iorlw   0xF0            ;w=H2-16
        movwf   D2              ;D2=H2-16
        addwf   D2,f            ;D2=H2*2-32
        addwf   D2,f            ;D2=H2*3-48
        movwf   D3              ;D3=H2-16
        addlw   -D'5'           ;w=H2-21
        addwf   D3,f            ;D3=H2*2-37 Done!
        addlw   D'41'           ;w=H2+20
        movwf   D1              ;D1=H2+20
        swapf   NumL,w
        iorlw   0xF0            ;w=H1-16
        addwf   D2,f            ;D2=H2*3+H1-64
        addwf   D1,f            ;D1=H2+H1+4, C=1
        rlf     D1,f            ;D1=(H2+H1)*2+9, C=0
        comf    D1,f            ;D1=-(H2+H1)*2-10
        rlf     D1,f            ;D1=-(H2+H1)*4-20
        movf    NumL,w
        andlw   0x0F            ;w=H0
        addwf   D1,f            ;D1=H0-(H2+H1)*4-20 Done!
        rlf     D2,f            ;C=0, D2=H2*6+H1*2-128 Done! 
        movlw   D'5'
        movwf   D4
        movlw   D'10'
  mod0
        addwf   D1,f            ;D(X)=D(X)mod10
        decf    D2,f            ;D(X+1)=D(X+1)+D(X)div10
        skpc
        goto    mod0
  mod1
        addwf   D2,f
        decf    D3,f
        skpc
        goto    mod1
  mod2
        addwf   D3,f
        decf    D4,f
        skpc
        goto    mod2
		movf	t_zn,w
		addwf	D4,f
        return
;/    "input" / 
Mult					
;copy accumulator to temporary
		clrf	t_zn		; "t_zn"														
		movf	NumH,w
		movwf	TEMP1
		movf	NumL,w
		movwf	TEMP0
;/  10- HEX-  5
;     Input: NumL .. NumH, 10 bits
;    Output: NumL .. NumH, 13 bits
; Code size: 16 instructions
; NumH_NumL*4	
		clrc
		rlf		TEMP0,f
		rlf		TEMP1,f
		rlf		TEMP0,f
		rlf		TEMP1,f	
;  NumH_NumL   TEMP1_TEMP0 
		movf	TEMP0,w
		addwf	NumL,f
		movf	TEMP1,w
		skpnc
		incfsz	TEMP1,w
		addwf	NumH,f
		btfss	NumH,4		; .  => 4 (  4095)
		goto	nRet
		movlw	4			
		movwf	t_zn		;  "4"  "t_zn"
		movlw	0x60		; 96  . 
		addwf	NumL,f
		skpc				; 
		goto	nRet
		incf	NumH		;  .   1
nRet
		return
; /
;/  16-   16 
;  "TEMP0",     "TEMP1"
Shift_16bit
		call	Shift_8bit		;   TEMP1   U1
		movf	TEMP0,w		;   TEMP0  TEMP1
		movwf	TEMP1
		call	Shift_8bit		;    U1   U2
;     TEMP1   U1
		bsf		PORTC,0		;     U1  U2
		nop
		nop
		bcf		PORTC,0
		return
Shift_8bit
		movlw	.8
		movwf	count		;   
per
		clrc	
		rrf		TEMP1,f
		skpnc
		goto	seg1		; =1
		bcf		PORTA,2		; =0
		goto	seg0
seg1	bsf		PORTA,2
seg0	bsf		PORTA,4		;  
		nop
		nop
		bcf		PORTA,4		;  
		DECFSZ	count,f		;  
		goto	per			;    	
		bcf		PORTA,2
		return
;/      
AdcPusk
		banksel	PIE1			;     
		bsf		PIE1, ADIE 		;   
		banksel	ADCON0			;   ADCON0
		comp	1,reg3,napr
		goto	tok
napr		movlw	081h			;  
		movwf 	ADCON0			;-AN0, Vref = VDD
		goto	zap_adc
tok		movlw	085h			;  
		movwf 	ADCON0			;-AN1, Vref = VDD
		movlw	.30
		call	Pause
zap_adc		bsf		ADCON0,GO		; ADC	
		return

;/  
ObrInt
		movwf	05eh
		movf	STATUS,w
		movwf	05fh	
	banksel		INTCON			;
		bcf		INTCON,GIE		;  
		btfss	PIE1, ADIE
		goto	it				;   
		btfss	PIR1,ADIF		;  
		goto	it				;   
		bcf		PIE1, ADIE 		;   
		bcf		PIR1,ADIF		;  ADIF
	banksel		ADRESL
		movf	ADRESL,w
		movwf	NumL			; . 
	banksel		ADRESH
		movf	ADRESH,w
		movwf	NumH			; . 
		goto	isr
it
	banksel		INTCON
		btfss	INTCON,T0IE		;  TMR0 ?
		goto	int_t1
		btfss	INTCON,T0IF		;
		goto	int_t1			;    0
		bcf		INTCON,T0IF		;  T0IF
		clrf	TMR0
		decf	cyc_tim,f		;   "TMR0"
		bsf		INTCON,T0IE		;   "TMR0"
		goto	isr				;  ,   
int_t1
		btfss	PIE1, TMR1IE
		goto	int_err			;  TMR1 
		btfss	PIR1,TMR1IF	
		goto	int_err
		bcf		PIR1,TMR1IF
		incfsz	T1L
		goto	it1
		incf	T1H
		goto	isr
it1			comp	2,T1L,isr
			bsf		D5,0
isr
		bsf		INTCON,GIE		; 
int_err							; 
		movf	05fh, w			; "STATUS"
		movwf	STATUS
		movf	05eh, w			; "W"
		retfie                  ;return from interrupt


			
;      RC 
			ORG   0x3FF 
           retlw   .128

; initialize eeprom locations
		
		ORG	0x2100
		DE	0x00, 0x01, 0x02, 0x03

		END                 ; directive 'end of program'
